#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

constexpr byte Char_Remain = Char_MyFighter;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}


static ptr<byte> PrintRemain(ptr<byte> pVram)
{
    byte c;
    c = Char_Remain;
    repeat (2) {
        repeat (2) {
            pVram = Put(pVram, c);
            ++c;
        }
        pVram += VramRowSize - 2 * VramStep;
    }
    return pVram + (2 * VramStep - VramRowSize * 2);
}


static ptr<byte> EraseRemain(ptr<byte> pVram)
{
    repeat (2) {
        repeat (2) {
            pVram = Put(pVram, Char_Space);
        }
        pVram += VramRowSize - 2 * VramStep;
    }
    return pVram + (2 * VramStep - VramRowSize * 2);
}


void PrintStatus() 
{
    PrintS(Vram + 1, "SCORE");
    PrintS(Vram + 8, "HI-SCORE");
    PrintS(Vram + 18, "STAGE");
    {
        ptr<byte> pVram;
        pVram = Vram + 27 * VramStep;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                pVram = PrintRemain(pVram);
                pVram = Put(pVram, Char_Space);
                pVram = Put(pVram, Char_Space);
                pVram += VramRowSize - 2 * VramStep;
                pVram = PrintC(pVram, i + 0x30);
                pVram = Put(pVram, Char_Space);
            }
            else {
                do {
                    pVram = PrintRemain(pVram);
                    --i;
                } while (i != 0);
                EraseRemain(pVram);
            }
        }
        else {
            EraseRemain(pVram);           
        }
    }
    PrintStage();
    PrintScore();
    // PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramRowSize, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramRowSize + 9 * VramStep, HiScore);
    PrintC(pVram, '0');
}

void PrintStage()
{
    PrintByteNumber2(Vram + VramRowSize + 20 * VramStep, CurrentStage + 1);
}

// void PrintTime()
// {
//     PrintByteNumber3(Vram + VramRowSize + 23, StageTime);
// }

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 11 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 11, "GAME OVER");
}

void PrintTimeUp() 
{
    EraseWindow();
    PrintS(Vram + VramRowSize * 12 + 12, "TIME UP");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	A
        0x1e, 0x2c, 0x2b, 0x20, 
        0x2a, 0x21, 0x1e, 0x2d, 
        0x2a, 0x25, 0x23, 0x2d, 
        0x22, 0x1f, 0x1e, 0x23, 
        //	E
        0x2a, 0x25, 0x23, 0x1f, 
        0x2a, 0x29, 0x28, 0x20, 
        0x2a, 0x21, 0x1e, 0x1e, 
        0x22, 0x23, 0x23, 0x1f, 
        //	R
        0x2a, 0x25, 0x23, 0x29, 
        0x2a, 0x21, 0x26, 0x2d, 
        0x2a, 0x25, 0x2d, 0x20, 
        0x22, 0x1f, 0x22, 0x23, 
        //	I
        0x22, 0x2b, 0x25, 0x1f, 
        0x1e, 0x2a, 0x21, 0x1e, 
        0x1e, 0x2a, 0x21, 0x1e, 
        0x22, 0x23, 0x23, 0x1f, 
        //	A
        0x1e, 0x2c, 0x2b, 0x20, 
        0x2a, 0x21, 0x1e, 0x2d, 
        0x2a, 0x25, 0x23, 0x2d, 
        0x22, 0x1f, 0x1e, 0x23, 
        //	L
        0x2a, 0x21, 0x1e, 0x1e, 
        0x2a, 0x21, 0x1e, 0x1e, 
        0x2a, 0x21, 0x1e, 0x1e, 
        0x22, 0x23, 0x23, 0x1f, 
    };

    ClearScreen(); 
    HideAllSprites();
    PrintStatus();
    {
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + (32 - 4 * 6) / 2;
        p = TitleBytes;
        repeat (6) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + 9, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 20 + 9, "OR RETURN KEY");
    PrintS(Vram + VramRowSize * 23 + VramWidth - 12, "INUFUTO 2023");
    // {
    //     byte c;
    //     word v;
    //     v = pVram+32*10;
    //     c = 0;
    //     repeat (0) {
    //         v = Put(v, c);
    //         ++c;
    //     }
    // }
}
